from bs4 import BeautifulSoup
from ..block import Block

class Section(Block):
    """

    Args:
        section_title (str):

    Attributes:
        section_title (str):

        content (dict):
    """

    def __init__(self,
                 section_title: str):

        self.section_title = section_title
        self.content = {}

    def set_content(self, content: dict):
        self.check_content(content)
        self.content = content

    def add_content(self, content: dict):
        self.check_content(content)
        self.content.update(content)

    def check_content(self, content: dict):
        assert type(content) == dict
        #for block in content.values():
        #    assert isinstance(block, Block), f"Each item in the content should be a Block, this item has type {type(block)}"

    def _as_html(self):
        html = f"<div><h1 class=\"section_title editable\">{self.section_title}</h1></div>"
        for item in self.content.values():
            html += item._as_html()

        return html



"""
Create html reports from python using simple objects

See module block for documentation on the blocks that can be added to reports.

Classes:
    Report
"""
from pathlib import Path
from datetime import datetime
from jinja2 import Template

from .block import Block


class Report:
    """
    Main .html report class.

    Args:
        target_folder (Path): Path to target folder, where to put report.

        template (Path): Path to report templates

        filename (str): Report filename

        template_content (dict): Extra template tags

    Attributes:
        target_folder (Path):

        template (Path):

        filename (str):

        report_header_info (dict):

        blocks (dict):
    """

    def __init__(self,
                 target_folder: Path = None,
                 template: Path = None,
                 filename: str = 'report',
                 template_content: dict = {}
                 ):
        self.template_content = template_content
        # To avoid long strings in documentation:
        if target_folder is None:
            self.target_folder = Path(__file__).parents[1]
        else:
            self.target_folder = target_folder

        if template is None:
            self.template = Path(__file__).parent.joinpath('templates/report_template_numerous.html')
        else:
            self.template = template

        assert '.html' not in filename, 'Input filename without suffix'
        self.filename = filename

        self.report_header_info = None
        self.blocks = {}

    def add_header_info(self,
                        header: str,
                        title: str,
                        sub_title: str,
                        sub_sub_title: str,
                        footer_title: str="",
                        footer_content: str=""
                        ):
        """
        Adding information to be used for populating html template

        Args:
            header (str): String with header

            title (str): String with title

            sub_title (str):  String with sub title

            sub_sub_title (str): String with sub sub title

        """

        self.report_header_info = dict(
            report_header=header,
            report_type_title=header,
            report_title=title,
            report_sub_title=sub_title,
            report_sub_sub_title=sub_sub_title,
            report_date="{: %d-%m-%Y}".format(datetime.now()),
            footer_title=footer_title,
            footer_content=footer_content
        )

    def set_blocks(self, blocks: dict):
        """
        Overriding blocks to the report.

        Args:
            blocks (dict): Content of block. Values in the key can either be an instance of the class Tabs or Section.

        """
        self._check_blocks(blocks)
        self.blocks = blocks

    def add_blocks(self, blocks: dict):
        """
        Appending blocks to the report.

        Args:
            blocks (dict): Content of block. Values in the key can either be an instance of the class Tabs or Section.

        """
        self._check_blocks(blocks)
        self.blocks.update(blocks)

    def _check_blocks(self, blocks: dict):
        assert type(blocks) == dict, "blocks should be a dictionary"
        for instance in blocks.values():
            assert isinstance(instance, Block), \
                "values in blocks should be Section or Tabs Class instances"

    def _as_html(self):
        """
        Writing objects and header info into html template

        Returns: String html

        """
        assert self.report_header_info is not None, "Please fill out header info"

        html = ""
        for content in self.blocks.values():
            html += content._as_html()

        with open(self.template, 'r') as tf:
            template_str = tf.read()

        t = Template(template_str)

        self.template_content.update(self.report_header_info)

        rendered = t.render(report_content=html, **self.template_content).encode('utf-8')

        return rendered

    def save(self):
        """Saving objects and header info as html"""
        with open(self.target_folder.joinpath(self.filename + '.html'), 'wb') as outfile:
            outfile.write(self._as_html())



from ..figure import Figure
from plotly import graph_objects as go
from plotly.basedatatypes import BaseTraceType
from typing import Union, Optional
def wrap_div(html, class_=None):
    if class_:
        return f"<div class='{class_}'>\n{html}\n</div>\n"
    else:
        return f'<div>\n{html}\n</div>\n'

class GoFigure(Figure):
    """

            Args:
                figure_data (dict):
                post_script (str):
            Attributes:
                html (str):
    """
    def __init__(self, figure_data: Union[list[BaseTraceType], dict], caption:str, notes:Optional[list[str]] = None, post_script: str = ""):
        super(GoFigure, self).__init__(caption, notes)
        self.figure_data = figure_data
        self.post_script = post_script
        self.figure_obj = go.Figure(self.figure_data)

    def _as_html_figure_content(self):

        html = wrap_div(self.figure_obj.to_html(config={"displayModeBar": True},
                                                            # show_link=False,
                                                            include_plotlyjs=False,
                                                            # output_type='div',
                                                            full_html=False,
                                                            post_script=self.post_script),
                        class_='figure_div')

        return html



from ..block import Block
from typing import Optional
from ..caption import Caption


class Figure(Block):

    def __init__(self, caption:str, notes: Optional[list[str]] = None):
        self.caption = Caption(caption=caption, notes=notes, type="Figure")

    def _as_html_figure_content(self):
        return ""

    def _as_html(self):
        return f"{self._as_html_figure_content() + self.caption.caption_as_html()}"




from typing import Optional

_figure_number = {}

def get_next_figure_number(type:str):
    if type not in _figure_number:
        _figure_number[type]=0
    _figure_number[type] += 1
    return _figure_number[type]

class Caption:

    def __init__(self, caption, notes: Optional[list[str]] = None, type: str = "Figure"):
        self.caption = caption
        if notes:
            self.notes = notes
        else:
            self.notes = []
        self.type = type
        self.number = get_next_figure_number(type)

    def caption_as_html(self):
        notes_str = ''.join([f'<div class="note"> <i>Note: {n} </i></div>' for n in self.notes])
        return f'<div class="caption-section"><div class="caption"><b>{self.type} {self.number}:</b> {self.caption}</div>{notes_str}</div>'


import setuptools
import os
from distutils.util import convert_path
from pathlib import Path

ver_path = convert_path('version.txt')
ver = None
if os.path.exists(ver_path):
    with open(ver_path) as ver_file:
        ver = ver_file.read()

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name="numerous html-report-generator",
    version=ver,
    author="numerous - Tobias Skov Reipuert, Tobias Dokkedal Elmøe, Lasse Nyberg Thomsen, Ósk Björnsdottir",
    author_email="report-generator@numerous.com",
    description="Report generator for html reports",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_namespace_packages(where='src'),
    package_dir={'': 'src'},
    package_data={"": ["*.html"]},
    include_package_data=True,
    install_requires=requirements,
    python_requires='>=3.9',
    license="O-BSD-3",
)




from ..block import Block
from typing import List
from ..caption import Caption


class Table(Block):

    def __init__(self, caption:str, notes: List[str]):
        self.caption = Caption(caption=caption, notes=notes, type="Table")



    # Overwrite this function in specific figure class
    def as_html_figure_content(self):
        return ""

    def _as_html(self):
        return f"{self.as_html_figure_content()+self.caption.caption_as_html()}"

